<?php namespace Premmerce\UsersRoles\Admin;

use Premmerce\UsersRoles\Models\AdminModel;
use Premmerce\UsersRoles\Behat\Transliterator\Transliterator;
use Premmerce\UsersRoles\WordpressSDK\FileManager\FileManager;

/**
 * Class Admin
 * @package Premmerce\UsersRoles\Admin
 */
class Admin{

	/**
	 * List with default roles
	 */
	const DEFAULT_ROLES = [
		'administrator',
		'editor',
		'author',
		'contributor',
		'subscriber',
	];

	const MENU_SLUG = 'premmerce-users-roles';

	/**
	 * @var FileManager
	 */
	private $fileManager;


	/**
	 * @var AdminModel
	 */
	private $model;

	/**
	 * List of all role capabilities
	 *
	 * @var array
	 */
	public $capabilities = [];

	/**
	 * Admin constructor.
	 *
	 * Register menu items and handlers
	 *
	 * @param FileManager $fileManager
	 */
	public function __construct(FileManager $fileManager){

		$this->fileManager = $fileManager;

		$this->model = new AdminModel();

		$this->capabilities = get_role('administrator')->capabilities;
		ksort($this->capabilities);

		add_action('init', function(){
			if(!session_id()){
				session_start();
			}
		});

		add_action('admin_menu', [$this, 'addMenuPage']);
		add_action('admin_menu', [$this, 'addFullPack'], 100);


		add_action('wp_ajax_getRoleCapabilities', [$this, 'getRoleCapabilities']);

		add_action('admin_post_premmerce_create_role', [$this, 'createRole']);
		add_action('admin_post_premmerce_update_role', [$this, 'updateRole']);
		add_action('admin_post_premmerce_delete_role', [$this, 'deleteRole']);
	}

	/**
	 * Add submenu to premmerce menu page
	 *
	 * @return false|string
	 */
	public function addMenuPage(){
		global $admin_page_hooks;

		$premmerceMenuExists = isset($admin_page_hooks['premmerce']);


		if(!$premmerceMenuExists){

			$svg = '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xml:space="preserve" width="20" height="16" style="fill:#82878c" viewBox="0 0 20 16"><g id="Rectangle_7"> <path d="M17.8,4l-0.5,1C15.8,7.3,14.4,8,14,8c0,0,0,0,0,0H8h0V4.3C8,4.1,8.1,4,8.3,4H17.8 M4,0H1C0.4,0,0,0.4,0,1c0,0.6,0.4,1,1,1 h1.7C2.9,2,3,2.1,3,2.3V12c0,0.6,0.4,1,1,1c0.6,0,1-0.4,1-1V1C5,0.4,4.6,0,4,0L4,0z M18,2H7.3C6.6,2,6,2.6,6,3.3V12 c0,0.6,0.4,1,1,1c0.6,0,1-0.4,1-1v-1.7C8,10.1,8.1,10,8.3,10H14c1.1,0,3.2-1.1,5-4l0.7-1.4C20,4,20,3.2,19.5,2.6 C19.1,2.2,18.6,2,18,2L18,2z M14,11h-4c-0.6,0-1,0.4-1,1c0,0.6,0.4,1,1,1h4c0.6,0,1-0.4,1-1C15,11.4,14.6,11,14,11L14,11z M14,14 c-0.6,0-1,0.4-1,1c0,0.6,0.4,1,1,1c0.6,0,1-0.4,1-1C15,14.4,14.6,14,14,14L14,14z M4,14c-0.6,0-1,0.4-1,1c0,0.6,0.4,1,1,1 c0.6,0,1-0.4,1-1C5,14.4,4.6,14,4,14L4,14z"/></g></svg>';
			$svg = 'data:image/svg+xml;base64,' . base64_encode($svg);

			add_menu_page(
				'Premmerce',
				'Premmerce',
				'manage_options',
				'premmerce',
				'',
				$svg
			);
		}

		$page = add_submenu_page(
			'premmerce',
			__('User roles', 'premmerce-users-roles'),
			__('User roles', 'premmerce-users-roles'),
			'manage_options',
			Admin::MENU_SLUG,
			[$this, 'controller']
		);

		if(!$premmerceMenuExists){
			global $submenu;
			unset($submenu['premmerce'][0]);
		}

		return $page;
	}

	public function addFullPack(){
		global $submenu;

		if(!function_exists('get_plugins')){
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugins = get_plugins();

		$premmerceInstalled = array_key_exists('premmerce-premium/premmerce.php', $plugins)
		                      || array_key_exists('premmerce/premmerce.php', $plugins);

		if(!$premmerceInstalled){
			$submenu['premmerce'][999] = [
				__('Get premmerce full pack', 'premmerce-users-roles'),
				'manage_options',
				admin_url('plugin-install.php?tab=plugin-information&plugin=premmerce'),
			];
		}
	}

	/**
	 * Control all actions in module
	 */
	public function controller(){
		$this->registerAssets();

		$this->controllerMessages();

		if(isset($_GET['edit_role'])){
			$this->controllerEditRole($_GET['edit_role']);
		}else{
			$this->controllerList();
		}
	}

	/**
	 * Control edit role
	 *
	 * @param string $editRole
	 */
	public function controllerEditRole($editRole){
		global $wp_roles;

		$errorMessage = null;

		if(!empty($editRole) && array_key_exists($editRole, $wp_roles->roles)){

			$this->fileManager->includeTemplate('admin/edit.php', [
				'curKey'          => $editRole,
				'curName'         => $wp_roles->roles[ $editRole ]['name'],
				'curCapabilities' => $wp_roles->roles[ $editRole ]['capabilities'],
				'roles'           => $wp_roles->roles,
				'capabilities'    => $this->capabilities,
				'dName'           => $this->getEnteredDisplayName(),
				'deleteUrl'       => $this->getDeleteUrl(),
			]);

		}else{
			$this->message([__('You attempted to edit an item that doesn’t exist. Perhaps it was deleted?', 'premmerce-users-roles')], 'error', []);
		}
	}

	/**
	 * Control roles list
	 */
	private function controllerList(){
		global $wp_roles;

		$current = isset($_GET['tab'])? $_GET['tab'] : 'list';

		$tabs['list'] = __('Roles list', 'premmerce-users-roles');

		if(function_exists('premmerce_re_fs')){
			$tabs['contact'] = __('Contact Us', 'premmerce-users-roles');
			if(premmerce_re_fs()->is_registered()){
				$tabs['account'] = __('Account', 'premmerce-users-roles');
			}
		}

		$this->fileManager->includeTemplate('admin/main.php', [
			'current'      => $current,
			'tabs'         => $tabs,
			'roles'        => $wp_roles->roles,
			'defaultRoles' => Admin::DEFAULT_ROLES,
			'editUrl'      => $this->getEditUrl(),
			'deleteUrl'    => $this->getDeleteUrl(),
			'dName'        => $this->getEnteredDisplayName(),
		]);
	}

	/**
	 * Control show error/complete messages
	 */
	private function controllerMessages(){
		if(isset($_SESSION['msg']) && !empty($_SESSION['msg'])){

			$this->message($_SESSION['msg'], $_SESSION['msgType'], isset($_SESSION['link'])? $_SESSION['link'] : []);

		}
	}

	/**
	 * Create new role and add capabilities for role
	 */
	public function createRole(){

		$displayName = $_POST['display_name'];
		$inheritRole = $_POST['role'];

		$roleName = $this->transliterate($displayName);
		$roleName = $this->checkRoleName($roleName);

		if($this->validation($roleName, $displayName, true)){

			$newCapabilities = [];

			if($inheritRole != 'null'){

				$role = $this->model->getRoles($inheritRole);

				if(!empty($role)){
					$newCapabilities = $role['capabilities'];
				}

			}

			add_role($roleName, $displayName, $newCapabilities);
		}

		wp_redirect($_SERVER['HTTP_REFERER']);
	}

	/**
	 * Delete role data
	 */
	public function deleteRole(){
		$redirectUrl = $this->getListUrl();

		if(isset($_GET['delete_role']) && !empty($_GET['delete_role'])){

			$roleName = $_GET['delete_role'];

			if(!in_array($roleName, Admin::DEFAULT_ROLES)){
				remove_role($roleName);

				$_SESSION['msg']     = [__('Role deleted. ', 'premmerce-users-roles')];
				$_SESSION['msgType'] = 'updated';
			}else{
				$redirectUrl = $this->getEditUrl() . $roleName;

				$_SESSION['msg']     = [__('Standard roles cannot be deleted or modified.', 'premmerce-users-roles')];
				$_SESSION['msgType'] = 'error';

				$this->setMessageBackToListLink();
			}
		}

		wp_redirect($redirectUrl);
	}

	/**
	 * Update role data
	 */
	public function updateRole(){
		$displayName = $_POST['display_name'];
		$roleName    = $_POST['role_name'];

		$capabilities = [];
		if(isset($_POST['capabilities'])){
			$capabilities = $_POST['capabilities'];
		}

		if($this->validation($roleName, $displayName)){

			$role = $this->model->getRoles($roleName);

			if(!empty($role)){

				$newCapabilities = [];
				foreach($capabilities as $c){
					$newCapabilities[ $c ] = true;
				}

				$role['name']         = $displayName;
				$role['capabilities'] = $newCapabilities;

				$this->setDBRole($roleName, $role);

				$_SESSION['msg']     = [__('Role updated.', 'premmerce-users-roles')];
				$_SESSION['msgType'] = 'updated';

				$this->setMessageBackToListLink();
			}

		}

		wp_redirect($_SERVER['HTTP_REFERER']);
	}

	/**
	 * Show messages
	 *
	 * @param array $msg
	 * @param string $type
	 * @param array $viewRole
	 */
	protected function message($msg = [], $type = 'updated', $viewRole = []){

		$link = '';
		if($viewRole){
			$link = ' <a href="' . $viewRole['url'] . '">' . $viewRole['text'] . '</a>';
		}

		foreach($msg as $m){
			echo '<div class="' . $type . '"><p>' . $m . $link . '</p></div>';
		}

		unset($_SESSION['msg'], $_SESSION['msgType'], $_SESSION['link']);
	}

	/**
	 *  Set link data to message
	 */
	private function setMessageBackToListLink(){
		$_SESSION['link'] = [
			'text' => '<p>&larr;' . __('Back to', 'premmerce-users-roles') . ' Premmerce Users Roles</p>',
			'url'  => $this->getListUrl(),
		];
	}

	/**
	 * Check entered data to valid
	 *
	 * @param string $roleName
	 * @param string $displayName
	 *
	 * @param bool $create
	 *
	 * @return bool
	 */
	protected function validation($roleName, $displayName, $create = false){

		global $wp_roles;

		$messages = [];

		if(in_array($roleName, Admin::DEFAULT_ROLES)){
			$messages[] = __('Standard roles cannot be deleted or modified.', 'premmerce-users-roles');

			$this->setMessageBackToListLink();
		}

		if(empty($displayName)){
			$messages[] = __('Name is empty. Please enter Name.', 'premmerce-users-roles');
		}

		if($create){
			if(array_search($displayName, $wp_roles->role_names)){
				$messages[] = __('Name already exist. Choose another Name.', 'premmerce-users-roles');
			}
		}else{
			foreach($wp_roles->role_names as $key => $value){

				if($displayName == $value && $roleName != $key){
					$messages[] = __('Name already exist. Choose another Name.', 'premmerce-users-roles');

					$_SESSION['display_name'] = $displayName;

					break;
				}

			}
		}

		if(count($messages)){

			$_SESSION['msg']     = $messages;
			$_SESSION['msgType'] = 'error';

		}else{
			return true;
		}
	}

	/**
	 * AJAX Return list of role capabiliries
	 */
	public function getRoleCapabilities(){

		global $wp_roles;

		$roleKey = $_POST['roleKey'];

		$role = [];
		if(isset($roleKey)){
			$role = $wp_roles->roles[ $roleKey ];
		}

		wp_send_json($role);

		wp_die();
	}

	/**
	 * Get URL for edit role
	 *
	 * @return string
	 */
	private function getListUrl(){
		return admin_url('admin.php') . '?page=' . Admin::MENU_SLUG;
	}

	/**
	 * Get URL for edit role
	 *
	 * @return string
	 */
	private function getEditUrl(){
		return admin_url('admin.php') . '?page=' . Admin::MENU_SLUG . '&edit_role=';
	}

	/**
	 * Get URL for delete role
	 *
	 * @return string
	 */
	private function getDeleteUrl(){
		return admin_url('admin-post.php') . '?action=premmerce_delete_role&delete_role=';
	}

	/**
	 * Save role data to BD
	 *
	 * @param string $roleName
	 * @param array $roleData
	 */
	private function setDBRole($roleName, $roleData = []){
		if(!empty($roleName) && !empty($roleData)){

			if(isset($roleData['name']) && isset($roleData['capabilities'])){

				$this->model->setRoles($roleName, $roleData);
			}
		}
	}

	/**
	 * Register assets for admin page
	 */
	private function registerAssets(){
		wp_enqueue_script(Admin::MENU_SLUG . '-script', $this->fileManager->locateAsset('admin/js/users-roles-script.js'));
		wp_enqueue_style(Admin::MENU_SLUG . '-style', $this->fileManager->locateAsset('admin/css/style.css'));
	}

	/**
	 * Check rle name and add ***-N if key exists
	 *
	 * @param $name
	 * @param int $i
	 *
	 * @return string
	 */
	private function checkRoleName($name, $i = 0){
		global $wp_roles;

		$n = $i > 0? $name . '-' . $i : $name;
		if(array_key_exists($n, $wp_roles->role_names)){
			return $this->checkRoleName($name, ++ $i);
		}else{
			return $n;
		}
	}

	/**
	 * Get transliterate word
	 *
	 * @param string $word
	 *
	 * @return string
	 */
	private function transliterate($word){
		return Transliterator::transliterate($word);
	}

	/**
	 * Get display name from session
	 *
	 * @return string
	 */
	private function getEnteredDisplayName(){

		$displayName = '';

		if(isset($_SESSION['display_name'])){
			$displayName = $_SESSION['display_name'];
			unset($_SESSION['display_name']);
		}

		return $displayName;
	}
}
