<?php namespace Premmerce\PriceTypes;

use Premmerce\PriceTypes\Admin\Admin;
use Premmerce\PriceTypes\Admin\AdminProducts;
use Premmerce\PriceTypes\Frontend\Frontend;
use Premmerce\PriceTypes\Models\AdminModel;
use Premmerce\PriceTypes\WordpressSDK\FileManager\FileManager;
use Premmerce\PriceTypes\WordpressSDK\Notifications\AdminNotifier;

/**
 * Class PriceTypesPlugin
 *
 * @package Premmerce\PriceTypes
 */
class PriceTypesPlugin
{

    const DOMAIN = 'premmerce-price-types';

    /**
     * @var FileManager
     */
    private $fileManager;

    /**
     * @var AdminModel
     */
    private $model;

    /**
     * @var AdminNotifier
     */
    private $notifier;

    /**
     * PluginManager constructor.
     *
     * @param $file
     */
    public function __construct($file)
    {
        $this->fileManager = new FileManager($file);
        $this->notifier = new AdminNotifier();

        $this->model = new AdminModel();

        add_action('init', [ $this, 'loadTextDomain' ]);
        add_action('admin_init', [ $this, 'checkRequirePlugins' ]);
    }

    /**
     * Run plugin part
     */
    public function run()
    {
        $valid = count($this->validateRequiredPlugins()) === 0;

        if (is_admin()) {
            new Admin($this->fileManager, $this->model);

            if ($valid) {
                new AdminProducts($this->fileManager, $this->model);
            }

        } elseif ($valid) {
            new Frontend($this->model);
        }
    }

    /**
     * Fired when the plugin is activated
     */
    public function activate()
    {
        $this->model->createTables();
    }

    /**
     * Fired during plugin uninstall
     */
    public static function uninstall()
    {
        $model = new AdminModel();
        $model->deleteTables();
    }

    /**
     * Check required plugins and push notifications
     */
    public function checkRequirePlugins(){
        $message = __('The %s plugin requires %s plugin to be active!', self::DOMAIN);

        $plugins = $this->validateRequiredPlugins();

        if(count($plugins)){
            foreach($plugins as $plugin){
                $error = sprintf($message, 'Premmerce Woocommerce Wholesale Pricing', $plugin);
                $this->notifier->push($error, AdminNotifier::ERROR, false);
            }
        }

    }

	/**
	 * Validate required plugins
	 *
	 * @return array
	 */
	private function validateRequiredPlugins(){

		$plugins = [];

		if(!function_exists('is_plugin_active')){
			include_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		/**
		 * Check if WooCommerce is active
		 **/
		if(!(is_plugin_active('woocommerce/woocommerce.php') || is_plugin_active_for_network('woocommerce/woocommerce.php'))){
			$plugins[] = '<a target="_blank" href="https://wordpress.org/plugins/woocommerce/">WooCommerce</a>';
		}

		return $plugins;
	}

    /**
     * Load plugin translations
     */
    public function loadTextDomain()
    {
        $name = $this->fileManager->getPluginName();
        load_plugin_textdomain('premmerce-price-types', false, $name . '/languages/');
    }
}
